@@ -2,7 +2,6 @@ context("Habitat associations")
 
 skip_on_cran()
 
-path <- ebirdst_download("example_data", tifs_only = FALSE)
 e <- ebirdst_extent(c(xmin = -86, xmax = -85, ymin = 42, ymax = 43))
 habitat <- ebirdst_habitat(path, ext = e)
 
@@ -10,13 +9,13 @@ test_that("ebirdst_habitat", {
   # expected
   expect_is(habitat, "ebirdst_habitat")
   expect_is(habitat, "data.frame")
-  expect_named(habitat, c("predictor", "date", "importance",  "prob_pos_slope",
-                          "direction"))
+  expect_named(habitat, c("predictor", "week", "importance",
+                          "prob_pos_slope", "direction"))
   expect_is(habitat$predictor, "character")
-  expect_is(habitat$date, "numeric")
+  expect_is(habitat$week, "character")
   expect_is(habitat$importance, "numeric")
   expect_is(habitat$direction, "numeric")
-  expect_length(unique(habitat$date), 52)
+  expect_length(unique(habitat$week), 52)
 
   # invalid inputs
   expect_error(ebirdst_habitat("/invalid/path", ext = e))
